@@ -21,30 +21,11 @@ import tkinter as tk
 import tkinter.ttk as ttk
 import traceback
 from tkinter import filedialog
-from pathlib import Path
 from mqttk.widgets.scroll_frame import ScrollFrame
 from mqttk.constants import SSL_LIST, MQTT_VERSION_LIST
 import uuid
 from functools import partial
-
-
-def validate_int(d, i, P, s, S, v, V, W):
-    try:
-        int(S)
-    except Exception:
-        return False
-    return True
-
-
-def validate_name(name, name_list):
-    if name not in name_list:
-        return name
-
-    template = name + " {}"
-    index = 1
-    while template.format(index) in name_list:
-        index += 1
-    return template.format(index)
+from mqttk.helpers import validate_name, validate_int
 
 
 class ConnectionFrame(ttk.Frame):
@@ -108,13 +89,9 @@ class ConfigurationWindow(tk.Toplevel):
         self.connections_frame.pack(side=tk.LEFT, anchor="w", fill="y", padx=3, pady=3)
         self.connections_listbox = ScrollFrame(self.connections_frame)
         self.connections_listbox.pack(fill='both', padx=3, pady=3, expand=1)
-        self.add_connection_button = ttk.Button(self.connections_frame)
-        self.add_connection_button["text"] = "Add"
+        self.add_connection_button = ttk.Button(self.connections_frame, text="Add", command=self.new_connection)
         self.add_connection_button.pack(padx=3, pady=3, side="right")
-        self.add_connection_button["command"] = self.new_connection
-        self.remove_connection_button = ttk.Button(self.connections_frame)
-        self.remove_connection_button["text"] = "Remove"
-        self.remove_connection_button["command"] = self.on_remove
+        self.remove_connection_button = ttk.Button(self.connections_frame, text="Remove", command=self.on_remove)
         self.remove_connection_button.pack(padx=3, pady=3, side="right")
 
         # Connection configuration frame
@@ -124,9 +101,7 @@ class ConfigurationWindow(tk.Toplevel):
         # Profile name
         self.profile_name_frame = ttk.Frame(self.connection_configuration_frame)
         self.profile_name_frame.pack(fill="x")
-        self.profile_label = ttk.Label(self.profile_name_frame, width=20)
-        self.profile_label["anchor"] = "e"
-        self.profile_label["text"] = "Profile name"
+        self.profile_label = ttk.Label(self.profile_name_frame, width=20, anchor="e", text="Profile name")
         self.profile_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
         self.profile_name_input = ttk.Entry(self.profile_name_frame)
         self.profile_name_input.pack(side=tk.LEFT, padx=2)
@@ -134,9 +109,7 @@ class ConfigurationWindow(tk.Toplevel):
         # Broker address
         self.broker_address_frame = ttk.Frame(self.connection_configuration_frame)
         self.broker_address_frame.pack(fill="x")
-        self.broker_label = ttk.Label(self.broker_address_frame, width=20)
-        self.broker_label["anchor"] = "e"
-        self.broker_label["text"] = "Broker address"
+        self.broker_label = ttk.Label(self.broker_address_frame, width=20, anchor="e", text="Broker address")
         self.broker_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
         self.broker_address_input = ttk.Entry(self.broker_address_frame, background="white")
         self.broker_address_input.pack(side=tk.LEFT, padx=2)
@@ -144,9 +117,7 @@ class ConfigurationWindow(tk.Toplevel):
         # Broker port
         self.broker_port_frame = ttk.Frame(self.connection_configuration_frame)
         self.broker_port_frame.pack(fill="x")
-        self.broker_port_label = ttk.Label(self.broker_port_frame, width=20)
-        self.broker_port_label["anchor"] = "e"
-        self.broker_port_label["text"] = "Broker port"
+        self.broker_port_label = ttk.Label(self.broker_port_frame, width=20, anchor="e", text="Broker port")
         self.broker_port_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
         self.broker_port_name_input = ttk.Entry(self.broker_port_frame)
         self.broker_port_name_input.configure(validate="all", validatecommand=vcmd)
@@ -155,23 +126,19 @@ class ConfigurationWindow(tk.Toplevel):
         # Client ID
         self.client_id_frame = ttk.Frame(self.connection_configuration_frame)
         self.client_id_frame.pack(fill="x")
-        self.client_id_label = ttk.Label(self.client_id_frame, width=20)
-        self.client_id_label["anchor"] = "e"
-        self.client_id_label["text"] = "Client ID"
+        self.client_id_label = ttk.Label(self.client_id_frame, width=20, anchor="e", text="Client ID")
         self.client_id_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
         self.client_id_input = ttk.Entry(self.client_id_frame)
         self.client_id_input.pack(side=tk.LEFT, padx=2)
-        self.client_id_generate_button = ttk.Button(self.client_id_frame)
-        self.client_id_generate_button["text"] = "Generate client ID"
-        self.client_id_generate_button["command"] = self.on_generate_client_id
+        self.client_id_generate_button = ttk.Button(self.client_id_frame,
+                                                    text="Generate client ID",
+                                                    command=self.on_generate_client_id)
         self.client_id_generate_button.pack(side=tk.LEFT, padx=2, pady=2)
 
         # username
         self.username_frame = ttk.Frame(self.connection_configuration_frame)
         self.username_frame.pack(fill="x")
-        self.username_label = ttk.Label(self.username_frame, width=20)
-        self.username_label["anchor"] = "e"
-        self.username_label["text"] = "Username"
+        self.username_label = ttk.Label(self.username_frame, width=20, anchor="e", text="Username")
         self.username_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
         self.username_input = ttk.Entry(self.username_frame)
         self.username_input.pack(side=tk.LEFT, padx=2)
@@ -179,9 +146,7 @@ class ConfigurationWindow(tk.Toplevel):
         # password
         self.password_frame = ttk.Frame(self.connection_configuration_frame)
         self.password_frame.pack(fill="x")
-        self.password_label = ttk.Label(self.password_frame, width=20)
-        self.password_label["anchor"] = "e"
-        self.password_label["text"] = "Password"
+        self.password_label = ttk.Label(self.password_frame, width=20, anchor="e", text="Password")
         self.password_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
         self.password_input = ttk.Entry(self.password_frame, show='*')
         self.password_input.pack(side=tk.LEFT, padx=2)
@@ -189,9 +154,7 @@ class ConfigurationWindow(tk.Toplevel):
         # timeout
         self.timeout_frame = ttk.Frame(self.connection_configuration_frame)
         self.timeout_frame.pack(fill="x")
-        self.timeout_label = ttk.Label(self.timeout_frame, width=20)
-        self.timeout_label["anchor"] = "e"
-        self.timeout_label["text"] = "Timeout"
+        self.timeout_label = ttk.Label(self.timeout_frame, width=20, anchor="e", text="Timeout")
         self.timeout_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
         self.timeout_input = ttk.Entry(self.timeout_frame)
         self.timeout_input.configure(validate="all", validatecommand=vcmd)
@@ -200,9 +163,7 @@ class ConfigurationWindow(tk.Toplevel):
         # keepalive
         self.keepalive_frame = ttk.Frame(self.connection_configuration_frame)
         self.keepalive_frame.pack(fill="x")
-        self.keepalive_label = ttk.Label(self.keepalive_frame, width=20)
-        self.keepalive_label["anchor"] = "e"
-        self.keepalive_label["text"] = "Keepalive"
+        self.keepalive_label = ttk.Label(self.keepalive_frame, width=20, anchor="e", text="Keepalive")
         self.keepalive_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
         self.keepalive_input = ttk.Entry(self.keepalive_frame)
         self.keepalive_input.configure(validate="all", validatecommand=vcmd)
@@ -211,69 +172,65 @@ class ConfigurationWindow(tk.Toplevel):
         # MQTT version
         self.version_frame = ttk.Frame(self.connection_configuration_frame)
         self.version_frame.pack(fill="x")
-        self.version_label = ttk.Label(self.version_frame, width=20)
-        self.version_label["anchor"] = "e"
-        self.version_label["text"] = "MQTT version"
+        self.version_label = ttk.Label(self.version_frame, width=20, anchor="e", text="MQTT version")
         self.version_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
-        self.version_input = ttk.Combobox(self.version_frame, exportselection=False)
+        self.version_input = ttk.Combobox(self.version_frame, exportselection=False, values=MQTT_VERSION_LIST)
         self.version_input.pack(side=tk.LEFT, padx=2)
-        self.version_input["values"] = MQTT_VERSION_LIST
         self.version_input.current(1)
         self.version_input["state"] = "readonly"
 
         # SSL
         self.ssl_state_frame = ttk.Frame(self.connection_configuration_frame)
         self.ssl_state_frame.pack(fill="x")
-        self.ssl_state_label = ttk.Label(self.ssl_state_frame, width=20)
-        self.ssl_state_label["anchor"] = "e"
-        self.ssl_state_label["text"] = "SSL"
+        self.ssl_state_label = ttk.Label(self.ssl_state_frame, width=20, anchor="e", text="SSL")
         self.ssl_state_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
-        self.ssl_state_input = ttk.Combobox(self.ssl_state_frame, exportselection=False)
+        self.ssl_state_input = ttk.Combobox(self.ssl_state_frame,
+                                            exportselection=False,
+                                            values=SSL_LIST,
+                                            state="readonly")
         self.ssl_state_input.pack(side=tk.LEFT, padx=2)
-        self.ssl_state_input["values"] = SSL_LIST
-
-        self.ssl_state_input["state"] = "readonly"
 
         # ca file
         self.ca_file_frame = ttk.Frame(self.connection_configuration_frame)
         self.ca_file_frame.pack(fill="x")
-        self.ca_file_label = ttk.Label(self.ca_file_frame, width=20)
-        self.ca_file_label["anchor"] = "e"
-        self.ca_file_label["text"] = "CA file"
+        self.ca_file_label = ttk.Label(self.ca_file_frame, width=20, anchor="e", text="CA file")
         self.ca_file_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
         self.ca_file_input = ttk.Entry(self.ca_file_frame, width=40)
         self.ca_file_input.pack(side=tk.LEFT, padx=2)
-        self.ca_browser_button = ttk.Button(self.ca_file_frame, width=3)
-        self.ca_browser_button["text"] = "..."
-        self.ca_browser_button["command"] = partial(self.browse_file, target_entry=self.ca_file_input)
+        self.ca_browser_button = ttk.Button(self.ca_file_frame,
+                                            width=3,
+                                            text="...",
+                                            command=partial(self.browse_file, target_entry=self.ca_file_input))
         self.ca_browser_button.pack(side=tk.LEFT, padx=2, pady=2)
 
         # client cert
         self.cl_cert_file_frame = ttk.Frame(self.connection_configuration_frame)
         self.cl_cert_file_frame.pack(fill="x")
-        self.cl_cert_file_label = ttk.Label(self.cl_cert_file_frame, width=20)
-        self.cl_cert_file_label["anchor"] = "e"
-        self.cl_cert_file_label["text"] = "Client certificate file"
+        self.cl_cert_file_label = ttk.Label(self.cl_cert_file_frame,
+                                            width=20,
+                                            anchor="e",
+                                            text="Client certificate file")
         self.cl_cert_file_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
         self.cl_cert_file_input = ttk.Entry(self.cl_cert_file_frame, width=40)
         self.cl_cert_file_input.pack(side=tk.LEFT, padx=2)
-        self.cl_cert_browser_button = ttk.Button(self.cl_cert_file_frame, width=3)
-        self.cl_cert_browser_button["text"] = "..."
-        self.cl_cert_browser_button["command"] = partial(self.browse_file, target_entry=self.cl_cert_file_input)
+        self.cl_cert_browser_button = ttk.Button(self.cl_cert_file_frame,
+                                                 width=3,
+                                                 text="...",
+                                                 command=partial(self.browse_file,
+                                                                 target_entry=self.cl_cert_file_input))
         self.cl_cert_browser_button.pack(side=tk.LEFT, padx=2, pady=2)
 
         # client_key
         self.cl_key_file_frame = ttk.Frame(self.connection_configuration_frame)
         self.cl_key_file_frame.pack(fill="x")
-        self.cl_key_file_label = ttk.Label(self.cl_key_file_frame, width=20)
-        self.cl_key_file_label["anchor"] = "e"
-        self.cl_key_file_label["text"] = "Client key file"
+        self.cl_key_file_label = ttk.Label(self.cl_key_file_frame, width=20, anchor="e", text="Client key file")
         self.cl_key_file_label.pack(side=tk.LEFT, anchor="w", padx=2, pady=4)
         self.cl_key_file_input = ttk.Entry(self.cl_key_file_frame, width=40)
         self.cl_key_file_input.pack(side=tk.LEFT, padx=2)
-        self.cl_key_browser_button = ttk.Button(self.cl_key_file_frame, width=3)
-        self.cl_key_browser_button["text"] = "..."
-        self.cl_key_browser_button["command"] = partial(self.browse_file, target_entry=self.cl_key_file_input)
+        self.cl_key_browser_button = ttk.Button(self.cl_key_file_frame,
+                                                width=3,
+                                                text="...",
+                                                command=partial(self.browse_file, target_entry=self.cl_key_file_input))
         self.cl_key_browser_button.pack(side=tk.LEFT, padx=2, pady=2)
 
         self.ssl_state_input.bind("<<ComboboxSelected>>", self.ssl_state_change)
@@ -282,15 +239,12 @@ class ConfigurationWindow(tk.Toplevel):
 
         self.button_frame = ttk.Frame(self.connection_configuration_frame)
         self.button_frame.pack(side='bottom', fill='x', anchor='s', expand=1)
-        self.ok_button = ttk.Button(self.button_frame, text="OK")
+        self.ok_button = ttk.Button(self.button_frame, text="OK", command=self.ok)
         self.ok_button.pack(side=tk.RIGHT, pady=5, padx=5)
-        self.ok_button["command"] = self.ok
-        self.apply_button = ttk.Button(self.button_frame, text="Apply")
+        self.apply_button = ttk.Button(self.button_frame, text="Apply", command=self.apply)
         self.apply_button.pack(side=tk.RIGHT, pady=5, padx=5)
-        self.apply_button["command"] = self.apply
-        self.cancel_button = ttk.Button(self.button_frame, text="Cancel")
+        self.cancel_button = ttk.Button(self.button_frame, text="Cancel", command=self.cancel)
         self.cancel_button.pack(side=tk.RIGHT, pady=5, padx=5)
-        self.cancel_button["command"] = self.cancel
 
         self.all_config_state_change("disabled")
 
@@ -355,10 +309,11 @@ class ConfigurationWindow(tk.Toplevel):
             self.connection_selected(self.currently_selected_connection)
 
     def browse_file(self, target_entry):
-        target_text = filedialog.askopenfilename(initialdir=str(Path.home()),
-                                                 title="Select CA file")
+        file_path_name = filedialog.askopenfilename(initialdir=self.config_handler.get_last_used_directory(),
+                                                    title="Select CA file")
+        self.config_handler.save_last_used_directory(file_path_name)
         target_entry.delete(0, tk.END)
-        target_entry.insert(0, target_text)
+        target_entry.insert(0, file_path_name)
 
     def add_profile_widget(self, connection_profile):
         self.profiles_widgets[connection_profile] = ConnectionFrame(self.connections_listbox.viewPort,
